from qtpy.QtWidgets import QMainWindow, QApplication, QWidget, QVBoxLayout, QSizePolicy, QMenu
from qtpy.QtGui import QMouseEvent, QResizeEvent
from qtpy.QtCore import Qt, QMetaObject, QTimer, Slot, QEvent, QObject

from .titlebar import Titlebar
from .titlebar.resizer import Resizer


class FramelessMainWindow(QMainWindow):
    """Frameless main window.

    Args:
        parent (QWidget, optional): Parent widget.
    """

    def __init__(self, parent=None):
        super(FramelessMainWindow, self).__init__(parent)
        self._contentWidgets = []
        self._enableResizing = True

        self.setWindowFlags(Qt.Window
                            | Qt.FramelessWindowHint
                            | Qt.WindowSystemMenuHint
                            | Qt.WindowCloseButtonHint
                            | Qt.WindowMinimizeButtonHint
                            | Qt.WindowMaximizeButtonHint)

        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setMouseTracking(True)

        # For preview
        screen = QApplication.desktop().availableGeometry()
        self.resize(screen.width() / 2, screen.height() / 2)
        self.setContentsMargins(0, 0, 0, 0)

        self._centralWidget = QWidget(self)
        self._centralWidget.setObjectName("centralWidget")
        self._centralWidget.setContentsMargins(2.5, 2.5, 2.5, 2.5)
        self._centralWidget.setMouseTracking(True)

        self._centralLayout = QVBoxLayout(self._centralWidget)
        self._centralLayout.setAlignment(Qt.AlignBottom)
        self._centralLayout.setSpacing(3)
        self._centralLayout.setContentsMargins(0, 0, 0, 0)

        self._bar = Titlebar(self)
        self._bar.showRestoreButton(False)
        self._bar.showLogoLabel(False)
        self._bar.showLogoButton(True)
        self._centralLayout.addWidget(self._bar)
        self._centralLayout.setAlignment(self._bar, Qt.AlignVCenter)

        self._centralLayout.addSpacing(3)

        self._contentWidget = QWidget(self)
        self._contentWidget.setMouseTracking(True)
        self._contentWidget.setSizePolicy(
            QSizePolicy.Expanding,
            QSizePolicy.Expanding)
        self._contentWidget.setContentsMargins(0, 0, 0, 0)
        self._contentWidgetLayout = QVBoxLayout(self._contentWidget)
        self._contentWidgetLayout.setContentsMargins(0, 0, 0, 0)
        self._centralLayout.addWidget(self._contentWidget)

        self._centralWidget.setLayout(self._centralLayout)
        self.setCentralWidget(self._centralWidget)

        self._fullscreenTitlebarTimer = QTimer(self)
        self._fullscreenTitlebarTimer.setObjectName("_fullscreenTitlebarTimer")

        self._contentWidget.setAutoFillBackground(True)

        self.minimizeClicked = self._bar.minimizeClicked
        self.maximizeClicked = self._bar.maximizeClicked
        self.restoreClicked = self._bar.restoreClicked
        self.closeClicked = self._bar.closeClicked

        # create resizer
        self.resizehandler = Resizer(self, debug=False)
        self.resizehandler.updateTitlebarHeight(self._bar.height())
        QApplication.instance().installEventFilter(self)

        QMetaObject.connectSlotsByName(self)

    def setMenu(self, menu: QMenu):
        """Set menu for app icon.

        Args:
            menu (QMenu): QMenu to show.
        """
        self._bar.setMenu(menu)

    def setTitlebarHeight(self, height: int):
        """Set titlebar height.

        Args:
            height (int): Titlebar height.
        """
        self._bar.setTitlebarHeight(height)
        self.resizehandler.updateTitlebarHeight(self._bar.height())

    def eventFilter(self, source, event):
        """Handles events. When in full screen mode the user places
        the cursor no more than five pixels from the top of the screen,
        a bar with buttons will appear. When the window is in NoState,
        the user is able to resize the window by hovering
        the cursor over the lower right corner of the window.

        Args:
            source (QObject): Event source.
            event (QEvent): Event.
        """

        # fullscreen hover to show buttons
        if self.windowState() == Qt.WindowFullScreen:
            if event.type() == QMouseEvent.MouseMove:
                ypos = event.globalY()
                if ypos <= 5:
                    self._bar.setVisible(True)
                    self._fullscreenTitlebarTimer.start(2500)
            else:
                self._fullscreenTitlebarTimer.stop()

        if hasattr(self, "resizehandler"):
            self.resizehandler.handle(source, event, self)
        return super().eventFilter(source, event)

    def resizeEvent(self, event: QResizeEvent) -> None:
        """Handle resize events

        Args:
            event (QResizeEvent): Resize event
        """
        pass

    def addContentWidget(self, widget: QWidget):
        """Add master widget to window.

        Args:
            widget (QWidget): Content widget.
        """
        self._contentWidgets.append(widget)
        self._contentWidgetLayout.addWidget(widget)

    def insertContentWidget(self, index, widget: QWidget):
        """Insert master widget to window at pos.

        Args:
            index (int): Index
            widget (QWidget): Content widget.
        """
        self._contentWidgets.insert(index, widget)
        self._contentWidgetLayout.insertWidget(index, widget)

    def showFullScreen(self) -> None:
        """Show app in fullscreen mode"""
        self.setWindowState(Qt.WindowFullScreen)

        firstshow_timer = QTimer(self)
        firstshow_timer.timeout.connect(self._on_showFullScreen)
        firstshow_timer.start(3000)

    def setResizingEnabled(self, value: bool):
        """Enable window resizing

        Args:
            value (bool): Enable or disable window resizing
        """
        self._enableResizing = value

    def isResizingEnabled(self) -> bool:
        """Return if window allows resizing

        Returns:
            value (bool): Window allow resizing.
        """
        return self._enableResizing

    def _on_showFullScreen(self):
        if self.windowState() == Qt.WindowFullScreen:
            self._bar.setVisible(False)

    @Slot()
    def on__fullscreenTitlebarTimer_timeout(self):
        if self.windowState() == Qt.WindowFullScreen:
            self._bar.setVisible(False)



html = """
<!DOCTYPE html>
<html>
<head>
    <script src="qrc:///qtwebchannel/qwebchannel.js"></script>
    <meta name="viewport" content="initial-scale=1.0, user-scalable=no">
    <meta charset="utf-8">
    <style>
        #map {
            height: 100%;
        }

        html, body {
            height: 100%;
            margin: 0;
            padding: 0;
        }
    </style>
</head>
<body>
<div id="map"></div>

<script async>

    var jshelper;
    new QWebChannel(qt.webChannelTransport,
                function(channel) { jshelper = channel.objects.jshelper; });
    let map;
    let markers = {};
    let polylines = {};

    const image = "https://developers.google.com/maps/documentation/javascript/examples/full/images/beachflag.png";
    var customStyle = [
        {
            "featureType": "water",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "COLOR_BACKGROUND_DARK"
                }
            ]
        },
        {
            "featureType": "landscape",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "COLOR_BACKGROUND_LIGHT"
                }
            ]
        },
        {
            "featureType": "road",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "#29768a"
                },
                {
                    "lightness": -37
                }
            ]
        },
        {
            "featureType": "poi",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "#406d80"
                }
            ]
        },
        {
            "featureType": "transit",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "#406d80"
                }
            ]
        },
        {
            "elementType": "labels.text.stroke",
            "stylers": [
                {
                    "visibility": "on"
                },
                {
                    "color": "COLOR_SELECTION_LIGHT"
                },
                {
                    "weight": 2
                },
                {
                    "gamma": 0.84
                }
            ]
        },
        {
            "elementType": "labels.text.fill",
            "stylers": [
                {
                    "color": "COLOR_FOREGROUND_LIGHT"
                }
            ]
        },
        {
            "featureType": "administrative",
            "elementType": "geometry",
            "stylers": [
                {
                    "weight": 0.6
                },
                {
                    "color": "#1a3541"
                }
            ]
        },
        {
            "elementType": "labels.icon",
            "stylers": [
                {
                    "visibility": "off"
                }
            ]
        },
        {
            "featureType": "poi.park",
            "elementType": "geometry",
            "stylers": [
                {
                    "color": "#2c5a71"
                }
            ]
        }
    ]

    function initMap() {
        map = new google.maps.Map(document.getElementById('map'), {
            zoom: 3,
            center: {lat: 0, lng: -180},
            disableDefaultUI: true,
            mapTypeId: google.maps.MapTypeId.ROADMAP,
            styles: customStyle,
            restriction: {
                latLngBounds: {
                    north: 85.0,
                    south: -85.0,
                    west: -179.9999,
                    east: 179.9999
                },
                strictBounds: true
            }
        });
        map.setOptions({minZoom: 0, maxZoom: 50});

        google.maps.event.addListener(map, "rightclick", function (event) {
            jshelper.mapIsRightClicked(event.latLng.lat(), event.latLng.lng())
        });

        google.maps.event.addListener(map, 'dragend', function () {
            var center = map.getCenter();
            jshelper.mapIsMoved(center.lat(), center.lng());
        });

        google.maps.event.addListener(map, 'click', function (event) {
            jshelper.mapIsClicked(event.latLng.lat(), event.latLng.lng());
        });

        google.maps.event.addListener(map, 'dblclick', function (event) {
            jshelper.mapIsDoubleClicked(event.latLng.lat(), event.latLng.lng());
        });

        console.log("Setup compeleted!");
    }


function addMarker(marker_id, latitude, longitude) {
    console.log("Creating new marker: " + marker_id)
    const location = new google.maps.LatLng(latitude, longitude);
    const marker = new google.maps.Marker({
        position: location,
        map: map,
        draggable: true,
        id: marker_id,
        polylines: {},
    });

    google.maps.event.addListener(marker, 'click', function () {
        jshelper.markerIsClicked(marker_id, marker.position.lat(), marker.position.lng())
    });
    google.maps.event.addListener(marker, 'dblclick', function () {
        jshelper.markerIsDoubleClicked(marker_id, marker.position.lat(), marker.position.lng())
    });
    google.maps.event.addListener(marker, 'rightclick', function () {
         jshelper.markerIsRightClicked(marker_id, marker.position.lat(), marker.position.lng())
    });
    google.maps.event.addListener(marker, 'dragend', function () {
         jshelper.markerIsMoved(marker_id, marker.position.lat(), marker.position.lng())
    });
    markers[marker_id] = marker;
}

function addPolyline(polyline_id, coordsArray) {
    console.log("Creating new polyline: " + polyline_id)
        const connection = new google.maps.Polyline({
        path: coordsArray,
        geodesic: true,
        strokeColor: "#FF0000",
        strokeOpacity: 1.0,
        strokeWeight: 2,
        map: map,
    });

    google.maps.event.addListener(connection, 'click', function () {
        jshelper.polylineIsClicked(polyline_id, connection.getPath().getArray())
    });
    google.maps.event.addListener(connection, 'dblclick', function () {
        jshelper.polylineIsDoubleClicked(polyline_id, connection.getPath().getArray())
    });
    google.maps.event.addListener(connection, 'rightclick', function () {
         jshelper.polylineIsRightClicked(polyline_id, connection.getPath().getArray())
    });
    polylines[polyline_id] = connection;
}

function getMarkers() {
    let resp = {};
    for (var mid in markers) {
        resp[mid] = {lat: markers[mid].position.lat(), lng: markers[mid].position.lng()};
    }
    return resp;
}

function moveMarker(marker_id, latitude, longitude) {
    /*
    var newcords = new google.maps.LatLng(latitude, longitude);
    for (var poly_id in polylines) {
        if (polylines[poly_id].get("markers").length > 0) {
            if(marker_id in polylines[poly_id].get("markers")) {
                var oldcords = polylines[poly_id].getPath().getArray();
                oldcords[marker_id] = newcords;
                polylines[poly_id].setPath(oldcords);
            }
        }
    }
    */
}

function deleteMarker(marker_id) {
    markers[marker_id].setMap(null);
    delete markers[marker_id];
    /*
    for (var poly_id in polylines) {
        if (polylines[poly_id].get("markers").length > 0) {
            if(marker_id in polylines[poly_id].get("markers")) {
                var oldcords = polylines[poly_id].getPath().getArray();
                oldcords.splice(marker_id, 1);
                polylines[poly_id]["markers"].splice(marker_id, 1);
                console.log("Removed marker with id: " + marker_id + "- New markersIDs for this poly: " + polylines[poly_id]["markers"])
                polylines[poly_id].setPath(oldcords);
            }
        }
    }
    */
}

function updateMarker(marker_id, extras) {
    if (!(marker_id in markers)) {
        return;
    }
    markers[marker_id].setOptions(extras);
}

</script>
<script async
        src="https://maps.googleapis.com/maps/api/js?key=API_KEY_GOES_HERE&callback=initMap">
</script>
</body>
</html>
"""



from qrainbowstyle.widgets.QtWaitingSpinner.pyqtspinner.spinner import WaitingSpinner
from qrainbowstyle.widgets.GoogleMapsWidget.MapsWidget import GoogleMapsView
from qrainbowstyle.widgets.PythonQtWidgets.qtwidgets.palette.palette import StylePicker
from qrainbowstyle.widgets.QRoundProgressBar.qroundprogressbar import QRoundProgressBar



# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'TestWidget.ui'
#
# Created by: PyQt5 UI code generator 5.8
#
# WARNING! All changes made in this file will be lost!

from qtpy import QtCore, QtGui, QtWidgets

from qroundprogressbar import QRoundProgressBar


class Ui_TestWidget(object):
    def setupUi(self, TestWidget):
        TestWidget.setObjectName("TestWidget")
        TestWidget.resize(703, 350)
        self.horizontalLayout = QtWidgets.QHBoxLayout(TestWidget)
        self.horizontalLayout.setContentsMargins(11, 11, 11, 11)
        self.horizontalLayout.setSpacing(6)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.gridLayout_3 = QtWidgets.QGridLayout()
        self.gridLayout_3.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_3.setSpacing(6)
        self.gridLayout_3.setObjectName("gridLayout_3")
        self.gridLayout_2 = QtWidgets.QGridLayout()
        self.gridLayout_2.setContentsMargins(11, 11, 11, 11)
        self.gridLayout_2.setSpacing(6)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.label_2 = QtWidgets.QLabel(TestWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_2.sizePolicy().hasHeightForWidth())
        self.label_2.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_2.setFont(font)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.gridLayout_2.addWidget(self.label_2, 0, 3, 1, 1)
        self.label = QtWidgets.QLabel(TestWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label.sizePolicy().hasHeightForWidth())
        self.label.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.gridLayout_2.addWidget(self.label, 0, 0, 1, 1)
        self.RoundBar1 = QRoundProgressBar(TestWidget)
        self.RoundBar1.setObjectName("RoundBar1")
        self.gridLayout_2.addWidget(self.RoundBar1, 2, 0, 1, 1)
        self.RoundBar2 = QRoundProgressBar(TestWidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(118, 118, 118))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(170, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(118, 118, 118))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(170, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(122, 122, 122))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(244, 244, 244))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(50, 100, 150))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Highlight, brush)
        self.RoundBar2.setPalette(palette)
        font = QtGui.QFont()
        font.setFamily("Courier New")
        font.setPointSize(18)
        font.setBold(True)
        font.setWeight(75)
        self.RoundBar2.setFont(font)
        self.RoundBar2.setObjectName("RoundBar2")
        self.gridLayout_2.addWidget(self.RoundBar2, 2, 1, 1, 1)
        self.RoundBar3 = QRoundProgressBar(TestWidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(170, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(170, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(244, 244, 244))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(50, 100, 150))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Highlight, brush)
        self.RoundBar3.setPalette(palette)
        self.RoundBar3.setObjectName("RoundBar3")
        self.gridLayout_2.addWidget(self.RoundBar3, 2, 3, 1, 1)
        self.label_3 = QtWidgets.QLabel(TestWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_3.sizePolicy().hasHeightForWidth())
        self.label_3.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setWeight(75)
        self.label_3.setFont(font)
        self.label_3.setAlignment(QtCore.Qt.AlignCenter)
        self.label_3.setObjectName("label_3")
        self.gridLayout_2.addWidget(self.label_3, 0, 1, 1, 1)
        self.gridLayout_3.addLayout(self.gridLayout_2, 0, 0, 1, 3)
        self.Slider = QtWidgets.QSlider(TestWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Ignored, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.Slider.sizePolicy().hasHeightForWidth())
        self.Slider.setSizePolicy(sizePolicy)
        self.Slider.setMaximum(999)
        self.Slider.setProperty("value", 150)
        self.Slider.setOrientation(QtCore.Qt.Horizontal)
        self.Slider.setObjectName("Slider")
        self.gridLayout_3.addWidget(self.Slider, 1, 0, 1, 3)
        self.RoundBar4 = QRoundProgressBar(TestWidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(170, 170, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(170, 170, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(244, 244, 244))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(50, 100, 150))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Highlight, brush)
        self.RoundBar4.setPalette(palette)
        self.RoundBar4.setObjectName("RoundBar4")
        self.gridLayout_3.addWidget(self.RoundBar4, 2, 0, 1, 1)
        self.RoundBar5 = QRoundProgressBar(TestWidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(170, 170, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(170, 170, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(244, 244, 244))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(50, 100, 150))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Highlight, brush)
        self.RoundBar5.setPalette(palette)
        self.RoundBar5.setObjectName("RoundBar5")
        self.gridLayout_3.addWidget(self.RoundBar5, 2, 1, 1, 1)
        self.RoundBar6 = QRoundProgressBar(TestWidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(170, 170, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(170, 170, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(244, 244, 244))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(50, 100, 150))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Highlight, brush)
        self.RoundBar6.setPalette(palette)
        self.RoundBar6.setObjectName("RoundBar6")
        self.gridLayout_3.addWidget(self.RoundBar6, 2, 2, 1, 1)
        self.horizontalLayout.addLayout(self.gridLayout_3)
        self.verticalLayout_3 = QtWidgets.QVBoxLayout()
        self.verticalLayout_3.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout_3.setSpacing(6)
        self.verticalLayout_3.setObjectName("verticalLayout_3")
        self.label_4 = QtWidgets.QLabel(TestWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Maximum)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.label_4.sizePolicy().hasHeightForWidth())
        self.label_4.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setPointSize(10)
        font.setBold(True)
        font.setUnderline(False)
        font.setWeight(75)
        self.label_4.setFont(font)
        self.label_4.setAlignment(QtCore.Qt.AlignCenter)
        self.label_4.setObjectName("label_4")
        self.verticalLayout_3.addWidget(self.label_4)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setContentsMargins(20, 11, 20, 11)
        self.horizontalLayout_2.setSpacing(6)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.RoundBar7 = QRoundProgressBar(TestWidget)
        palette = QtGui.QPalette()
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(57, 57, 57))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Button, brush)
        brush = QtGui.QBrush(QtGui.QColor(85, 85, 85))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Light, brush)
        brush = QtGui.QBrush(QtGui.QColor(71, 71, 71))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Midlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(28, 28, 28))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Dark, brush)
        brush = QtGui.QBrush(QtGui.QColor(38, 38, 38))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Mid, brush)
        brush = QtGui.QBrush(QtGui.QColor(68, 68, 68))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Text, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.BrightText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Shadow, brush)
        brush = QtGui.QBrush(QtGui.QColor(100, 44, 104))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 220))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ToolTipBase, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Active, QtGui.QPalette.ToolTipText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(57, 57, 57))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Button, brush)
        brush = QtGui.QBrush(QtGui.QColor(85, 85, 85))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Light, brush)
        brush = QtGui.QBrush(QtGui.QColor(71, 71, 71))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Midlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(28, 28, 28))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Dark, brush)
        brush = QtGui.QBrush(QtGui.QColor(38, 38, 38))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Mid, brush)
        brush = QtGui.QBrush(QtGui.QColor(68, 68, 68))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Text, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.BrightText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Shadow, brush)
        brush = QtGui.QBrush(QtGui.QColor(100, 44, 104))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 220))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ToolTipBase, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Inactive, QtGui.QPalette.ToolTipText, brush)
        brush = QtGui.QBrush(QtGui.QColor(28, 28, 28))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.WindowText, brush)
        brush = QtGui.QBrush(QtGui.QColor(57, 57, 57))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Button, brush)
        brush = QtGui.QBrush(QtGui.QColor(85, 85, 85))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Light, brush)
        brush = QtGui.QBrush(QtGui.QColor(71, 71, 71))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Midlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(28, 28, 28))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Dark, brush)
        brush = QtGui.QBrush(QtGui.QColor(38, 38, 38))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Mid, brush)
        brush = QtGui.QBrush(QtGui.QColor(28, 28, 28))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Text, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 255))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.BrightText, brush)
        brush = QtGui.QBrush(QtGui.QColor(28, 28, 28))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ButtonText, brush)
        brush = QtGui.QBrush(QtGui.QColor(57, 57, 57))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Base, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Shadow, brush)
        brush = QtGui.QBrush(QtGui.QColor(50, 100, 150))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.Highlight, brush)
        brush = QtGui.QBrush(QtGui.QColor(255, 255, 220))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ToolTipBase, brush)
        brush = QtGui.QBrush(QtGui.QColor(0, 0, 0))
        brush.setStyle(QtCore.Qt.SolidPattern)
        palette.setBrush(QtGui.QPalette.Disabled, QtGui.QPalette.ToolTipText, brush)
        self.RoundBar7.setPalette(palette)
        self.RoundBar7.setObjectName("RoundBar7")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.RoundBar7)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.horizontalLayout_2.addWidget(self.RoundBar7)
        self.verticalLayout_3.addLayout(self.horizontalLayout_2)
        self.horizontalLayout.addLayout(self.verticalLayout_3)

        self.retranslateUi(TestWidget)
        QtCore.QMetaObject.connectSlotsByName(TestWidget)

    def retranslateUi(self, TestWidget):
        _translate = QtCore.QCoreApplication.translate
        TestWidget.setWindowTitle(_translate("TestWidget", "TestWidget"))
        self.label_2.setText(_translate("TestWidget", "Line Style"))
        self.label.setText(_translate("TestWidget", "Donut Style"))
        self.label_3.setText(_translate("TestWidget", "Pie Style"))
        self.label_4.setText(_translate("TestWidget", "Donut Style"))



import logging

from qtpy.QtCore import QObject, QPoint, QRect, Qt
from qtpy.QtGui import QMouseEvent
from qtpy.QtWidgets import QApplication, QWidget


class FramelessApplication(QApplication):
    def __init__(self, *args):
        super(FramelessApplication, self).__init__(*args)

    def notify(self, source, event) -> bool:
        if hasattr(source, "setMouseTracking"):
            source.setMouseTracking(True)
        return QApplication.notify(source, event)


class Resizer(QObject):
    """Resizer object handles frameless windows resizing events

    Note:
        Why not use eventFilter? - After closing application,
        broken reference to window may cause "underlying C/C++
        object has been deleted" error.

    Args:
        gripsize (int): Side length of grip detection area.
        dndarea (QPoint): Do Not Detect area starting from right bottom corner.
        sidegripdnd (int): Do Not Detect side length for side grips.
    """

    def __init__(self, parent=None, debug=False, gripsize=4, sidegripdnd=20, titlebarheight=50):
        super(Resizer, self).__init__(parent)
        self._debug = debug

        self._titlebarHeight = titlebarheight
        self._gripsize = gripsize
        self._sidegripdnd = sidegripdnd
        self._griprect = None
        self._hgriprect = None
        self._vgriprect = None

        self._resizing = False
        self._hor_resizing = False
        self._ver_resizing = False

        self._logger = logging.getLogger(__name__)

    def _updateGripRect(self, window):
        """Update rects for current window geometry.

        Args:
            window (QWidget): Source window
        """
        # corner grip
        self._griprect = QRect(window.width() - self._gripsize,
                               window.height() - self._gripsize,
                               self._gripsize,
                               self._gripsize)

        # right grip
        self._hgriprect = QRect(window.width() - self._gripsize,
                                self._titlebarHeight,
                                self._gripsize,
                                window.height() - self._gripsize - self._sidegripdnd - self._titlebarHeight)

        # bottom grip
        self._vgriprect = QRect(self._gripsize,
                                window.height() - self._gripsize,
                                window.width() - self._gripsize - self._sidegripdnd,
                                self._gripsize)

    def updateTitlebarHeight(self, height):
        self._titlebarHeight = height

    def handle(self, source, event, parent):
        """Handle frameless window resizing events.

        Args:
            source (QObject): Event source.
            event (QEvent): Event.
            parent (QWidget): Window to alter
        """

        # resizing
        parent.deleteLater()
        try:
            self._updateGripRect(parent)
            try:
                if hasattr(source, "window") and source.window() is parent:
                    if self._debug:
                        self._logger.debug(
                            "Accepting event from: " + str(source) + "\nEvent type is: " + str(event.type()))
                    if parent.isResizingEnabled() and parent.windowState() not in (
                            Qt.WindowFullScreen, Qt.WindowMaximized):
                        # button pressed
                        if event.type() == QMouseEvent.MouseButtonPress:
                            self._updateGripRect(parent)
                            if self._griprect.contains(
                                    event.pos()) and not self._hor_resizing and not self._ver_resizing:
                                # corner grip
                                self._resizing = True

                            elif self._hgriprect.contains(
                                    event.pos()) and not self._resizing and not self._ver_resizing:
                                # right grip
                                self._hor_resizing = True

                            elif self._vgriprect.contains(
                                    event.pos()) and not self._resizing and not self._hor_resizing:
                                # bottom grip
                                self._ver_resizing = True

                        if event.type() == QMouseEvent.MouseButtonRelease:
                            # stop resizing
                            if self._resizing or self._hor_resizing or self._ver_resizing:
                                QApplication.restoreOverrideCursor()
                                self._resizing = False
                                self._hor_resizing = False
                                self._ver_resizing = False
                                QApplication.restoreOverrideCursor()
                                self._updateGripRect(parent)

                        if event.type() == QMouseEvent.MouseMove:
                            # moving
                            self._updateGripRect(parent)
                            if self._resizing and not self._hor_resizing and not self._ver_resizing:
                                if event.buttons() == Qt.LeftButton:
                                    QApplication.setOverrideCursor(Qt.SizeFDiagCursor)
                                    parent.resize(event.x(), event.y())
                                    self._updateGripRect(parent)
                                    parent.update()
                                    QApplication.restoreOverrideCursor()

                            elif self._hor_resizing and not self._resizing and not self._ver_resizing:
                                if event.buttons() == Qt.LeftButton:
                                    QApplication.setOverrideCursor(Qt.SizeHorCursor)
                                    parent.resize(event.x(), parent.height())
                                    self._updateGripRect(parent)
                                    parent.update()
                                    QApplication.restoreOverrideCursor()

                            elif self._ver_resizing and not self._resizing and not self._hor_resizing:
                                if event.buttons() == Qt.LeftButton:
                                    QApplication.setOverrideCursor(Qt.SizeVerCursor)
                                    parent.resize(parent.width(), event.y())
                                    self._updateGripRect(parent)
                                    parent.update()
                                    QApplication.restoreOverrideCursor()

                            else:
                                if self._griprect.contains(
                                        event.pos()) and not self._hor_resizing and not self._ver_resizing:
                                    QApplication.setOverrideCursor(Qt.SizeFDiagCursor)
                                elif self._hgriprect.contains(
                                        event.pos()) and not self._resizing and not self._ver_resizing:
                                    QApplication.setOverrideCursor(Qt.SizeHorCursor)
                                elif self._vgriprect.contains(
                                        event.pos()) and not self._resizing and not self._hor_resizing:
                                    QApplication.setOverrideCursor(Qt.SizeVerCursor)
                                else:
                                    QApplication.restoreOverrideCursor()
            except Exception as e:
                if self._debug:
                    self._logger.critical(e)

        except Exception as e:
            self._logger.critical(e)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#######################################################
#
# Copyright 2017 Pete Alexandrou
#
# Ported to Python from the original works in C++ by:
#
#     Sintegrial Technologies (c) 2015
#     https://sourceforge.net/projects/qroundprogressbar
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
#######################################################

import sys

from qtpy.QtCore import Qt
from qtpy.QtGui import QPalette
from qtpy.QtWidgets import QApplication, QWidget

from qroundprogressbar import QRoundProgressBar
from ui_TestWidget import Ui_TestWidget


class TestWidget(QWidget, Ui_TestWidget):
    def __init__(self, parent=None):
        super(TestWidget, self).__init__(parent)
        self.setupUi(self)

        self.RoundBar1.setFormat('%v')
        self.RoundBar1.setDecimals(0)
        self.connectToSlider(self.RoundBar1)

        self.RoundBar2.setNullPosition(QRoundProgressBar.PositionRight)
        self.RoundBar2.setBarStyle(QRoundProgressBar.BarStyle.PIE)
        self.connectToSlider(self.RoundBar2)

        self.RoundBar3.setFormat('%m')
        self.RoundBar3.setBarStyle(QRoundProgressBar.BarStyle.LINE)
        self.connectToSlider(self.RoundBar3)

        p1 = QPalette()
        p1.setBrush(QPalette.AlternateBase, Qt.black)
        p1.setColor(QPalette.Text, Qt.yellow)
        self.RoundBar4.setPalette(p1)
        self.RoundBar4.setNullPosition(QRoundProgressBar.PositionLeft)
        self.RoundBar4.setDecimals(0)
        gradientPoints = [(0, Qt.green), (0.5, Qt.yellow), (1, Qt.red)]
        self.RoundBar4.setDataColors(gradientPoints)
        self.connectToSlider(self.RoundBar4)

        p2 = QPalette(p1)
        p2.setBrush(QPalette.Base, Qt.lightGray)
        p2.setColor(QPalette.Text, Qt.magenta)
        p2.setColor(QPalette.Shadow, Qt.green)
        self.RoundBar5.setPalette(p2)
        self.RoundBar5.setNullPosition(QRoundProgressBar.PositionRight)
        self.RoundBar5.setBarStyle(QRoundProgressBar.BarStyle.PIE)
        self.RoundBar5.setDataColors(gradientPoints)
        self.connectToSlider(self.RoundBar5)

        self.RoundBar6.setDecimals(2)
        self.RoundBar6.setBarStyle(QRoundProgressBar.BarStyle.LINE)
        self.RoundBar6.setOutlinePenWidth(18)
        self.RoundBar6.setDataPenWidth(10)
        self.connectToSlider(self.RoundBar6)

        self.connectToSlider(self.RoundBar7)

    def connectToSlider(self, bar: QRoundProgressBar):
        bar.setRange(self.Slider.minimum(), self.Slider.maximum())
        bar.setValue(self.Slider.value())
        self.Slider.valueChanged.connect(bar.setValue)


if __name__ == '__main__':
    app = QApplication(sys.argv)
    widget = TestWidget()
    widget.show()
    sys.exit(app.exec_())



from qtpy.QtWidgets import QApplication, QWidget, QVBoxLayout, QSizePolicy, QDialog
from qtpy.QtGui import QResizeEvent, QMouseEvent
from qtpy.QtCore import Qt, QMetaObject, QEvent

from .titlebar import Titlebar
from .titlebar.resizer import Resizer


class FramelessDialog(QDialog):
    """FramelessDialog documentation"""

    def __init__(self, parent=None):
        super(FramelessDialog, self).__init__(parent)
        self._contentWidgets = []
        self._enableResizing = True

        self.setWindowFlags(Qt.Window
                            | Qt.FramelessWindowHint
                            | Qt.WindowSystemMenuHint
                            | Qt.WindowCloseButtonHint
                            | Qt.WindowMinimizeButtonHint
                            | Qt.WindowMaximizeButtonHint)

        self.setAttribute(Qt.WA_TranslucentBackground)
        self.setMouseTracking(True)

        # For preview
        screen = QApplication.desktop().availableGeometry()
        self.resize(screen.width() / 2, screen.height() / 2)
        self.setContentsMargins(0, 0, 0, 0)

        self._centralWidget = QWidget(self)
        self._centralWidget.setObjectName("centralWidget")
        self._centralWidget.setContentsMargins(2.5, 2.5, 2.5, 2.5)
        self._centralWidget.setMouseTracking(True)

        self._centralLayout = QVBoxLayout(self._centralWidget)
        self._centralLayout.setAlignment(Qt.AlignBottom)
        self._centralLayout.setSpacing(3)
        self._centralLayout.setContentsMargins(0, 0, 0, 0)

        self._bar = Titlebar(self)
        self._bar.showRestoreButton(False)
        self._bar.showMinimizeButton(False)
        self._bar.showMaximizeButton(False)
        self._bar.showLogoLabel(True)
        self._bar.showLogoButton(False)
        self._bar.maximizeOnDoubleClick(False)
        self._bar.enableEdgesSnapping(False)
        self._centralLayout.addWidget(self._bar)
        self._centralLayout.setAlignment(self._bar, Qt.AlignVCenter)

        self._centralLayout.addSpacing(3)

        self._contentWidget = QWidget(self)
        self._contentWidget.setSizePolicy(
            QSizePolicy.Expanding,
            QSizePolicy.Expanding)
        self._contentWidget.setContentsMargins(0, 0, 0, 0)
        self._contentWidgetLayout = QVBoxLayout(self._contentWidget)
        self._contentWidgetLayout.setContentsMargins(0, 0, 0, 0)
        self._centralLayout.addWidget(self._contentWidget)

        self._centralWidget.setLayout(self._centralLayout)
        self._contentWidget.setAutoFillBackground(True)

        self.minimizeClicked = self._bar.minimizeClicked
        self.closeClicked = self._bar.closeClicked

        self._layout = QVBoxLayout(self)
        self._layout.setContentsMargins(0, 0, 0, 0)
        self.setLayout(self._layout)
        self._layout.addWidget(self._centralWidget)

        # create resizer
        self.resizehandler = Resizer(debug=False)
        QApplication.instance().installEventFilter(self)

        QMetaObject.connectSlotsByName(self)

    def setTitlebarHeight(self, height: int):
        """Set titlebar height.

        Args:
            height (int): Titlebar height.
        """
        self._bar.setTitlebarHeight(height)

    def eventFilter(self, source, event):
        """Handles events. When in full screen mode the user places
        the cursor no more than five pixels from the top of the screen,
        a bar with buttons will appear. When the window is in NoState,
        the user is able to resize the window by hovering
        the cursor over the lower right corner of the window.

        Args:
            source (QObject): Event source.
            event (QEvent): Event.
        """

        self.resizehandler.handle(source, event, self)
        return QDialog.eventFilter(self, source, event)

    def addContentWidget(self, widget: QWidget):
        """Add master widget to window.

        Args:
            widget (QWidget): Content widget.
        """
        self._contentWidgets.append(widget)
        self._contentWidgetLayout.addWidget(widget)

    def insertContentWidget(self, index, widget: QWidget):
        """Insert master widget to window at pos.

        Args:
            index (int): Index
            widget (QWidget): Content widget.
        """
        self._contentWidgets.insert(index, widget)
        self._contentWidgetLayout.insertWidget(index, widget)

    def setResizingEnabled(self, value: bool):
        """Enable window resizing

        Args:
            value (bool): Enable or disable window resizing
        """
        self._enableResizing = value

    def isResizingEnabled(self) -> bool:
        """Return if window allows resizing

        Returns:
            value (bool): Window allow resizing.
        """
        return self._enableResizing



from .framelessdialog import FramelessDialog
from .framelessmainwindow import FramelessMainWindow
from .framelessmessagebox import (FramelessMessageBox, FramelessCriticalMessageBox, FramelessInformationMessageBox,
                                  FramelessQuestionMessageBox, FramelessWarningMessageBox)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""GoogleMapsView allows to load Google Maps to Qt app.

Usage

.. code-block:: python

    import qrainbowstyle
    import qrainbowstyle.widgets

Work in progress...
"""
import json

from qtpy.QtCore import QObject, Slot, QEvent, Signal
from qtpy.QtWebEngineWidgets import QWebEngineView, QWebEngineSettings, QWebEnginePage
from qtpy.QtNetwork import QNetworkProxyFactory
from qtpy.QtWebChannel import QWebChannel

import qrainbowstyle
import logging

from .mapHTML import html


def _replace_api_key(api_key: str):
    return html.replace("API_KEY_GOES_HERE", api_key)


class CallHandler(QObject):
    """Google Maps events handler. Emits signals for
    map, markers, polylines - clicked, doubleClicked, rightClicked,
    moved events.

    Args:
        parent (object): Parent object.
    """
    runJavascript = Signal(str, object)

    mapMoved = Signal(float, float)
    mapClicked = Signal(float, float)
    mapRightClicked = Signal(float, float)
    mapDoubleClicked = Signal(float, float)

    markerMoved = Signal(str, float, float)
    markerClicked = Signal(str, float, float)
    markerDoubleClicked = Signal(str, float, float)
    markerRightClicked = Signal(str, float, float)

    polylineClicked = Signal(str, list)
    polylineRightClicked = Signal(str, list)
    polylineDoubleClicked = Signal(str, list)

    def __init__(self, parent=None):
        super(CallHandler, self).__init__(parent)
        self._logger = logging.getLogger(__name__)
        self.markers = []

    def runScript(self, script, callback=None):
        """Run Javascript code.

        Args:
            script (str): Script to execute.
            callback (callback, optional): Function to handle callback.
        """
        self.runJavascript.emit(script, callback)

    @Slot(float, float)
    def mapIsRightClicked(self, lat, lng):
        """Handle right clicks on map.

        Args:
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Right clicked on: " + str(lat) + ":" + str(lng))
        self.mapRightClicked.emit(lat, lng)

    @Slot(float, float)
    def mapIsMoved(self, lat, lng):
        """Handle moving the map event.

        Args:
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Moved, center: " + str(lat) + ":" + str(lng))
        self.mapMoved.emit(lat, lng)

    @Slot(float, float)
    def mapIsClicked(self, lat, lng):
        """Handle clicks on map.

        Args:
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Clicked on: " + str(lat) + ":" + str(lng))
        self.mapClicked.emit(lat, lng)

    @Slot(float, float)
    def mapIsDoubleClicked(self, lat, lng):
        """Handle double clicks on map.

        Args:
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Double clicked on: " + str(lat) + ":" + str(lng))
        self.mapDoubleClicked.emit(lat, lng)

    @Slot(str, float, float)
    def markerIsClicked(self, marker_id, lat, lng):
        """Handle clicks on markers.

        Args:
            marker_id (str): Marker id.
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Marker " + str(marker_id) + " clicked: " + str(lat) + ":" + str(lng))
        self.markerClicked.emit(marker_id, lat, lng)

    @Slot(str, float, float)
    def markerIsRightClicked(self, marker_id, lat, lng):
        """Handle right clicks on markers.

        Args:
            marker_id (str): Marker id.
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Marker " + str(marker_id) + " right clicked: " + str(lat) + ":" + str(lng))
        self.markerRightClicked.emit(marker_id, lat, lng)

    @Slot(str, float, float)
    def markerIsDoubleClicked(self, marker_id, lat, lng):
        """Handle double clicks on markers.

        Args:
            marker_id (str): Marker id.
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Marker " + str(marker_id) + " double clicked: " + str(lat) + ":" + str(lng))
        self.markerDoubleClicked.emit(marker_id, lat, lng)
        self.deleteMarker(marker_id)

    @Slot(str, float, float)
    def markerIsMoved(self, marker_id, lat, lng):
        """Handle moving markers.

        Args:
            marker_id (str): Marker id.
            lat (float): Event latitude.
            lng (float): Event longitude.
        """
        self._logger.debug("Marker " + str(marker_id) + " moved to: " + str(lat) + ":" + str(lng))
        self.moveMarker(marker_id, lat, lng)

    @Slot(str, list)
    def polylineIsClicked(self, polyline_id, path: list):
        """Handle clicks on polylines.

        Args:
            polyline_id (str): Polyline id.
            path (list): List of coordinates.
        """
        self._logger.debug("Polyline " + str(polyline_id) + " clicked: " + str(path))
        self.polylineClicked.emit(polyline_id, path)

    @Slot(str, list)
    def polylineIsRightClicked(self, polyline_id, path: list):
        """Handle right clicks on polylines.

        Args:
            polyline_id (str): Polyline id.
            path (list): List of coordinates.
        """
        self._logger.debug("Polyline " + str(polyline_id) + " right clicked: " + str(path))
        self.polylineRightClicked.emit(polyline_id, path)

    @Slot(str, list)
    def polylineIsDoubleClicked(self, polyline_id, path: list):
        """Handle double clicks on polylines.

        Args:
            polyline_id (str): Polyline id.
            path (list): List of coordinates.
        """
        self._logger.debug("Polyline " + str(polyline_id) + " double clicked: " + str(path))
        self.polylineDoubleClicked.emit(polyline_id, path)

    def _updateMarkersCallback(self, markers: list):
        """Callback for loading marker list from QWebEngineView."""
        self.markers = markers

    def loadMarkers(self):
        """Updates markers list."""
        self.runScript("getMarkers();", self._updateMarkersCallback)

    def updateMarker(self, marker_id, args):
        """Update markers parameters."""
        return self.runScript("updateMarker({}, {});".format(marker_id, json.dumps(args)))

    def moveMarker(self, marker_id, latitude, longitude):
        """Move marker to provided latitude and longitude."""
        return self.runScript("moveMarker({}, {}, {});".format(marker_id, latitude, longitude))

    def deleteMarker(self, marker_id):
        """Delete marker with provided ID."""
        return self.runScript("deleteMarker({});".format(marker_id))

    def addMarker(self, marker_id, latitude, longitude):
        """Creates marker with marker_id id at latitude, longitude.

        Args:
            marker_id (float): Marker id.
            latitude (float): Marker latitude.
            longitude (float): Marker longitude.
        """
        return self.runScript("addMarker({}, {}, {});".format(marker_id, latitude, longitude))

    def addPolyline(self, polyline_id, coordinates: list):
        """Creates polyline between coordinates.

        Args:
            polyline_id (str): Polyline ID.
            coordinates (list): List of coordinates (dicts with "lat" and "lng" keys).
        """
        return self.runScript("addPolyline({}, {});".format(polyline_id, coordinates))

    def addPolylineBetweenMarkers(self, polyline_id, markers_ids: list):
        """Creates polyline between markers.

        Args:
            polyline_id (str): Polyline ID.
            markers_ids (list): List of markers IDs.
        """
        return self.runScript("addPolylineBetweenMarkers({}, {});".format(polyline_id, markers_ids))


class GoogleMapsPage(QWebEnginePage):
    """QWebEngineView page for handling Javascript console messages."""

    def __init__(self, parent=None):
        super(GoogleMapsPage, self).__init__(parent)
        self._logger = logging.getLogger(__name__)

    def javaScriptConsoleMessage(self, level, msg, line, source_id):
        """Handle Javascript console messages.

        Args:
            level (int): Logging level.
            msg (str): Message string.
            line (int): Line in code where error occured.
            source_id (str): Element ID.
        """
        self._logger.log(10 * (level + 1), "[{}]: {}".format(line, msg))


class GoogleMapsView(QWebEngineView):
    """Show Google Maps in Qt app."""

    def __init__(self, parent, api_key):
        super(GoogleMapsView, self).__init__(parent)
        self._logger = logging.getLogger(__name__)

        # Set browser attributes
        QNetworkProxyFactory.setUseSystemConfiguration(False)
        self.settings().setAttribute(QWebEngineSettings.Accelerated2dCanvasEnabled, True)
        self.settings().setAttribute(QWebEngineSettings.JavascriptEnabled, True)

        self.api_key = api_key

        # Create maps page (it is not needed, but we can handle Javascript console logs)
        self._mapspage = GoogleMapsPage(self)
        self.setPage(self._mapspage)

        # Create connection between Javascript and Qt
        self.channel = QWebChannel(self.page())
        self.page().setWebChannel(self.channel)

        # create map events handler and register it as "jshelper" in HTML
        self.handler = CallHandler(self)
        self.handler.runJavascript.connect(self.runScript)
        self.channel.registerObject("jshelper", self.handler)

        # Set HTML
        self.setHtml(qrainbowstyle.rainbowize(_replace_api_key(self.api_key)))

    @Slot(str, object)
    def runScript(self, script, callback):
        """Run Javascript code.

        Args:
            script (str): Script to execute.
            callback (callback, optional): Function to handle callback.
        """
        if callback is None:
            self.page().runJavaScript(script)
        else:
            self.page().runJavaScript(script, callback)

    def addMarker(self, marker_id, lat, lng):
        """Creates marker with marker_id id at latitude, longitude.

        Args:
            marker_id (float): Marker id.
            lat (float): Marker latitude.
            lng (float): Marker longitude.
        """
        self.handler.addMarker(marker_id, lat, lng)

    def addPolyline(self, polyline_id, markers: list):
        """Creates polyline using coordinates.

        Args:
            polyline_id (str): Polyline id
            markers (list): List of coordinates (dicts with "lat" and "lng" keys).
        """
        self.handler.addPolyline(polyline_id, markers)

    def changeEvent(self, event: QEvent):
        """Change event handler.

        Args:
            event (QEvent): Event.
        """
        if event.type() == QEvent.StyleChange:
            self.setHtml(qrainbowstyle.rainbowize(_replace_api_key(self.api_key)))



from qtpy.QtCore import Qt, QSize
from qtpy.QtWidgets import QGridLayout, QLabel, QStyle, QDialogButtonBox, QSizePolicy, QWidget, QApplication
from qtpy.QtGui import QIcon

from .framelessdialog import FramelessDialog


class FramelessMessageBox(FramelessDialog):
    """FramelessMessageBox documentation"""

    def __init__(self, icon=None, parent=None):
        super(FramelessMessageBox, self).__init__(parent)
        self.setResizingEnabled(False)
        self.resize(350, 150)

        self._messagewidget = QWidget(self)
        self._messagewidget.setContentsMargins(11, 5, 11, 11)

        self._grid = QGridLayout(self._messagewidget)
        self._grid.setContentsMargins(0, 0, 0, 0)
        self._grid.setVerticalSpacing(8)
        self._grid.setHorizontalSpacing(0)

        self._messagewidget.setLayout(self._grid)

        self._iconLabel = QLabel(self._messagewidget)
        self._iconLabel.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        self._iconLabel.setScaledContents(True)
        self._grid.addWidget(self._iconLabel, 0, 0, 2, 1, Qt.AlignTop | Qt.AlignLeft)

        self._textLabel = QLabel(self._messagewidget)
        self._textLabel.setWordWrap(True)
        self._textLabel.setContentsMargins(2, 0, 0, 0)
        self._textLabel.setIndent(9)
        self._textLabel.setScaledContents(True)
        self._textLabel.setMouseTracking(True)
        self._textLabel.autoFillBackground()
        self._textLabel.setSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)

        self._grid.addWidget(self._textLabel, 0, 1, 1, 1)

        self._grid.setRowStretch(1, 100)
        self._grid.setRowMinimumHeight(2, 6)

        self.buttonBox = QDialogButtonBox(self._messagewidget)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Ok | QDialogButtonBox.Cancel)
        self._grid.addWidget(self.buttonBox, 3, 1, 1, 1)

        self.addContentWidget(self._messagewidget)
        self.setIcon(icon)

    def setStandardButtons(self, buttons):
        self.buttonBox.setStandardButtons(buttons)

    def addButton(self, button, role):
        self.buttonBox.addButton(button, role)

    def removeButton(self, button):
        self.buttonBox.removeButton(button)

    def buttons(self):
        return self.buttonBox.buttons()

    def setText(self, text: str):
        self._textLabel.setText(text)
        self._textLabel.adjustSize()
        self._textLabel.updateGeometry()

        policy = QSizePolicy(QSizePolicy.Minimum, QSizePolicy.Minimum)
        policy.setHeightForWidth(True)
        self._textLabel.setSizePolicy(policy)
        self.setSizePolicy(policy)
        self.adjustSize()

    def text(self):
        return self._textLabel.text()

    def setIcon(self, icon):
        if icon:
            icon = QApplication.style().standardIcon(icon)
            self._iconLabel.setPixmap(icon.pixmap(QSize(128, 128)))
        else:
            self._iconLabel.setPixmap(QIcon().pixmap(QSize(128, 128)))


class FramelessWarningMessageBox(FramelessMessageBox):
    def __init__(self, parent=None):
        super(FramelessWarningMessageBox, self).__init__(icon=QStyle.SP_MessageBoxWarning, parent=parent)


class FramelessInformationMessageBox(FramelessMessageBox):
    def __init__(self, parent=None):
        super(FramelessInformationMessageBox, self).__init__(icon=QStyle.SP_MessageBoxInformation, parent=parent)


class FramelessCriticalMessageBox(FramelessMessageBox):
    def __init__(self, parent=None):
        super(FramelessCriticalMessageBox, self).__init__(icon=QStyle.SP_MessageBoxCritical, parent=parent)


class FramelessQuestionMessageBox(FramelessMessageBox):
    def __init__(self, parent=None):
        super(FramelessQuestionMessageBox, self).__init__(icon=QStyle.SP_MessageBoxQuestion, parent=parent)
